@@ -34,6 +34,7 @@
 #include <sys/stat.h>
 
 #include "debugger.h"
+#include "preferences.h"
 
 #include <libanjuta/interfaces/ianjuta-debugger.h>
 #include <libanjuta/interfaces/ianjuta-debugger-breakpoint.h>
@@ -42,6 +43,7 @@
 #include <libanjuta/interfaces/ianjuta-debugger-instruction.h>
 #include <libanjuta/interfaces/ianjuta-debugger-variable.h>
 #include <libanjuta/interfaces/ianjuta-terminal.h>
+#include <libanjuta/interfaces/ianjuta-preferences.h>
 #include <libanjuta/anjuta-plugin.h>
 #include <signal.h>
 
@@ -64,6 +66,9 @@ struct _GdbPlugin
 
 	/* Terminal */
 	pid_t term_pid;
+
+	/* Pretty printer list */
+	GList *pretty_printers;
 };
 
 struct _GdbPluginClass
@@ -74,7 +79,8 @@ struct _GdbPluginClass
 /* Terminal functions
  *---------------------------------------------------------------------------*/
 
-#define PREF_TERMINAL_COMMAND "anjuta.command.terminal"
+#define PREF_SCHEMA "org.gnome.anjuta.run"
+#define PREF_TERMINAL_COMMAND "run-terminal-command"
 
 static void
 gdb_plugin_stop_terminal (GdbPlugin* plugin)
@@ -109,7 +115,7 @@ gdb_plugin_start_terminal (GdbPlugin* plugin)
 	if (mkfifo (file, 0664) < 0)
 	{
 		anjuta_util_dialog_error (GTK_WINDOW (ANJUTA_PLUGIN (plugin)->shell),
-					  _("Failed to create fifo file named %s. The program will run without a terminal."), file);
+					  _("Failed to create FIFO file named %s. The program will run without a terminal."), file);
 		g_free (file);
 
 		return NULL;
@@ -123,16 +129,14 @@ gdb_plugin_start_terminal (GdbPlugin* plugin)
 	if (term == NULL)
 	{
 		/* Use gnome terminal or another user defined one */
-		AnjutaPreferences *pref;
+		GSettings* settings;
 		gchar *term_cmd;
 		gchar **argv;
 		
-		pref = anjuta_shell_get_preferences (ANJUTA_PLUGIN (plugin)->shell, NULL);
-		term_cmd = anjuta_preferences_get (pref, PREF_TERMINAL_COMMAND);
-		if (!term_cmd)
-		{
-			term_cmd = g_strdup ("gnome-terminal --disable-factory -e %s");
-		}
+		settings = g_settings_new (PREF_SCHEMA);
+
+		term_cmd = g_settings_get_string (settings, PREF_TERMINAL_COMMAND);
+		g_object_unref (settings);
 		if (g_shell_parse_argv (term_cmd, NULL, &argv, NULL))
 		{
 			GPid gpid;
@@ -234,6 +238,58 @@ gdb_plugin_initialize (GdbPlugin *this)
 	g_signal_connect_swapped (this, "debugger-stopped", G_CALLBACK (on_debugger_stopped), this);
 	debugger_set_output_callback (this->debugger, this->output_callback, this->output_user_data);
 	if (this->view) debugger_set_log (this->debugger, this->view);
+	
+	debugger_set_pretty_printers (this->debugger, this->pretty_printers);
+}
+
+/* Helper functions
+ *---------------------------------------------------------------------------*/
+
+static gchar *
+quote_expression (const gchar *expression) {
+	GRegex *regex;
+	gchar *aux, *aux2;
+	gchar *quoted;
+	
+	/* This part is for expressions with an actual \" inside */
+	regex = g_regex_new ("\\\\\"", G_REGEX_MULTILINE, 0, NULL);
+	aux = g_regex_replace_literal (regex, expression, -1, 0, "\\\\\\\"", 0, NULL);
+	g_regex_unref (regex);
+
+	/* This part is for expressions with an " inside */
+	regex = g_regex_new ("(?<!\\\\)\"", G_REGEX_MULTILINE, 0, NULL);
+	aux2 = g_regex_replace_literal (regex, aux, -1, 0, "\\\"", 0, NULL);
+	g_regex_unref (regex);
+	g_free (aux);
+
+	/* Converts newlines in spaces */
+	g_strdelimit (aux2, "\n", ' ');
+	quoted = g_strconcat ("\"",aux2, "\"", NULL); 
+	g_free (aux2);
+	
+	return quoted;
+}
+
+/* Callback for saving session
+ *---------------------------------------------------------------------------*/
+
+static void
+on_session_save (AnjutaShell *shell, AnjutaSessionPhase phase, AnjutaSession *session, GdbPlugin *this)
+{
+	if (phase != ANJUTA_SESSION_PHASE_NORMAL)
+		return;
+
+	gdb_save_pretty_printers (session, this->pretty_printers);
+}
+
+static void on_session_load (AnjutaShell *shell, AnjutaSessionPhase phase, AnjutaSession *session, GdbPlugin *this)
+{
+	if (phase != ANJUTA_SESSION_PHASE_NORMAL)
+		return;
+
+	g_list_foreach (this->pretty_printers, (GFunc)gdb_pretty_printer_free, NULL);
+	g_list_free (this->pretty_printers);
+	this->pretty_printers = gdb_load_pretty_printers (session);
 }
 
 /* AnjutaPlugin functions
@@ -242,10 +298,18 @@ gdb_plugin_initialize (GdbPlugin *this)
 static gboolean
 gdb_plugin_activate_plugin (AnjutaPlugin* plugin)
 {
-	/* GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin); */
+	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
 	DEBUG_PRINT ("%s", "GDB: Activating Gdb plugin...");
+	this->pretty_printers = NULL;
 
+	/* Connect to session signal */
+	g_signal_connect (plugin->shell, "save-session",
+					  G_CALLBACK (on_session_save), this);
+	g_signal_connect (plugin->shell, "load-session",
+					  G_CALLBACK (on_session_load), this);
+	
+	
 	return TRUE;
 }
 
@@ -262,6 +326,10 @@ gdb_plugin_deactivate_plugin (AnjutaPlugin* plugin)
 		this->debugger = NULL;
 	}
 	
+	g_list_foreach (this->pretty_printers, (GFunc)gdb_pretty_printer_free, NULL);
+	g_list_free (this->pretty_printers);
+	this->pretty_printers = NULL;
+	
 	return TRUE;
 }
 
@@ -563,7 +631,7 @@ idebugger_interrupt (IAnjutaDebugger *plugin, GError **err)
 }
 
 static gboolean
-idebugger_inspect (IAnjutaDebugger *plugin, const gchar *name, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
+idebugger_inspect (IAnjutaDebugger *plugin, const gchar *name, IAnjutaDebuggerGCharCallback callback, gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 	
@@ -573,7 +641,7 @@ idebugger_inspect (IAnjutaDebugger *plugin, const gchar *name, IAnjutaDebuggerCa
 }
 
 static gboolean
-idebugger_evaluate (IAnjutaDebugger *plugin, const gchar *name, const gchar *value, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
+idebugger_evaluate (IAnjutaDebugger *plugin, const gchar *name, const gchar *value, IAnjutaDebuggerGCharCallback callback, gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 	gchar* buf;
@@ -596,7 +664,7 @@ idebugger_send_command (IAnjutaDebugger *plugin, const gchar* command, GError **
 }
 
 static gboolean
-idebugger_print (IAnjutaDebugger *plugin, const gchar* variable, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
+idebugger_print (IAnjutaDebugger *plugin, const gchar* variable, IAnjutaDebuggerGCharCallback callback, gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -606,7 +674,7 @@ idebugger_print (IAnjutaDebugger *plugin, const gchar* variable, IAnjutaDebugger
 }
 
 static gboolean
-idebugger_list_local (IAnjutaDebugger *plugin, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
+idebugger_list_local (IAnjutaDebugger *plugin, IAnjutaDebuggerGListCallback callback , gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -616,7 +684,7 @@ idebugger_list_local (IAnjutaDebugger *plugin, IAnjutaDebuggerCallback callback
 }
 
 static gboolean
-idebugger_list_argument (IAnjutaDebugger *plugin, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
+idebugger_list_argument (IAnjutaDebugger *plugin, IAnjutaDebuggerGListCallback callback , gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -626,7 +694,7 @@ idebugger_list_argument (IAnjutaDebugger *plugin, IAnjutaDebuggerCallback callba
 }
 
 static gboolean
-idebugger_info_signal (IAnjutaDebugger *plugin, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
+idebugger_info_signal (IAnjutaDebugger *plugin, IAnjutaDebuggerGListCallback callback , gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -636,7 +704,7 @@ idebugger_info_signal (IAnjutaDebugger *plugin, IAnjutaDebuggerCallback callback
 }
 
 static gboolean
-idebugger_info_sharedlib (IAnjutaDebugger *plugin, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
+idebugger_info_sharedlib (IAnjutaDebugger *plugin, IAnjutaDebuggerGListCallback callback , gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -659,7 +727,7 @@ idebugger_handle_signal (IAnjutaDebugger *plugin, const gchar* name, gboolean st
 }
 
 static gboolean
-idebugger_info_frame (IAnjutaDebugger *plugin, guint frame, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
+idebugger_info_frame (IAnjutaDebugger *plugin, guint frame, IAnjutaDebuggerGListCallback callback , gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -669,7 +737,7 @@ idebugger_info_frame (IAnjutaDebugger *plugin, guint frame, IAnjutaDebuggerCallb
 }
 
 static gboolean
-idebugger_info_args (IAnjutaDebugger *plugin, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
+idebugger_info_args (IAnjutaDebugger *plugin, IAnjutaDebuggerGListCallback callback , gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -679,7 +747,7 @@ idebugger_info_args (IAnjutaDebugger *plugin, IAnjutaDebuggerCallback callback ,
 }
 
 static gboolean
-idebugger_info_target (IAnjutaDebugger *plugin, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
+idebugger_info_target (IAnjutaDebugger *plugin, IAnjutaDebuggerGListCallback callback , gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -689,7 +757,7 @@ idebugger_info_target (IAnjutaDebugger *plugin, IAnjutaDebuggerCallback callback
 }
 
 static gboolean
-idebugger_info_program (IAnjutaDebugger *plugin, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
+idebugger_info_program (IAnjutaDebugger *plugin, IAnjutaDebuggerGListCallback callback , gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -699,7 +767,7 @@ idebugger_info_program (IAnjutaDebugger *plugin, IAnjutaDebuggerCallback callbac
 }
 
 static gboolean
-idebugger_info_udot (IAnjutaDebugger *plugin, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
+idebugger_info_udot (IAnjutaDebugger *plugin, IAnjutaDebuggerGListCallback callback , gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -709,7 +777,7 @@ idebugger_info_udot (IAnjutaDebugger *plugin, IAnjutaDebuggerCallback callback ,
 }
 
 static gboolean
-idebugger_info_variables (IAnjutaDebugger *plugin, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
+idebugger_info_variables (IAnjutaDebugger *plugin, IAnjutaDebuggerGListCallback callback , gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -729,7 +797,7 @@ idebugger_set_frame (IAnjutaDebugger *plugin, guint frame, GError **err)
 }
 
 static gboolean
-idebugger_list_frame (IAnjutaDebugger *plugin, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
+idebugger_list_frame (IAnjutaDebugger *plugin, IAnjutaDebuggerGListCallback callback , gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -749,7 +817,7 @@ idebugger_set_thread (IAnjutaDebugger *plugin, gint thread, GError **err)
 }
 
 static gboolean
-idebugger_list_thread (IAnjutaDebugger *plugin, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
+idebugger_list_thread (IAnjutaDebugger *plugin, IAnjutaDebuggerGListCallback callback , gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -759,7 +827,7 @@ idebugger_list_thread (IAnjutaDebugger *plugin, IAnjutaDebuggerCallback callback
 }
 
 static gboolean
-idebugger_info_thread (IAnjutaDebugger *plugin, gint thread, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
+idebugger_info_thread (IAnjutaDebugger *plugin, gint thread, IAnjutaDebuggerGListCallback callback , gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -769,11 +837,21 @@ idebugger_info_thread (IAnjutaDebugger *plugin, gint thread, IAnjutaDebuggerCall
 }
 
 static gboolean
-idebugger_list_register (IAnjutaDebugger *plugin, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
+idebugger_run_from (IAnjutaDebugger *plugin, const gchar *file, gint line, GError **err)
 {
-	//GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
+	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
+	
+	debugger_run_from_position (this->debugger, file, line);
 
-	//debugger_list_register (this->debugger, callback, user_data);
+	return TRUE;
+}
+
+static gboolean
+idebugger_dump_stack_trace (IAnjutaDebugger *plugin, IAnjutaDebuggerGListCallback callback , gpointer user_data, GError **err)
+{
+	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
+
+	debugger_dump_stack_trace (this->debugger, callback, user_data);
 
 	return TRUE;
 }
@@ -825,6 +903,7 @@ idebugger_iface_init (IAnjutaDebuggerIface *iface)
 	iface->step_over = idebugger_step_over;
 	iface->step_out = idebugger_step_out;
 	iface->run_to = idebugger_run_to;
+	iface->run_from = idebugger_run_from;
 	iface->exit = idebugger_exit;
 	iface->interrupt = idebugger_interrupt;
 
@@ -848,7 +927,7 @@ idebugger_iface_init (IAnjutaDebuggerIface *iface)
 	iface->list_thread = idebugger_list_thread;
 	iface->set_thread = idebugger_set_thread;
 	iface->info_thread = idebugger_info_thread;
-	iface->list_register = idebugger_list_register;
+	iface->dump_stack_trace = idebugger_dump_stack_trace;
 
 	iface->send_command = idebugger_send_command;
 
@@ -874,7 +953,7 @@ idebugger_breakpoint_implement (IAnjutaDebuggerBreakpoint *plugin, GError **err)
 }
 
 static gboolean
-idebugger_breakpoint_add_at_line (IAnjutaDebuggerBreakpoint *plugin, const gchar* file, guint line, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
+idebugger_breakpoint_add_at_line (IAnjutaDebuggerBreakpoint *plugin, const gchar* file, guint line, IAnjutaDebuggerBreakpointCallback callback, gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 	
@@ -884,7 +963,7 @@ idebugger_breakpoint_add_at_line (IAnjutaDebuggerBreakpoint *plugin, const gchar
 }
 
 static gboolean
-idebugger_breakpoint_add_at_function (IAnjutaDebuggerBreakpoint *plugin, const gchar* file, const gchar* function, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
+idebugger_breakpoint_add_at_function (IAnjutaDebuggerBreakpoint *plugin, const gchar* file, const gchar* function, IAnjutaDebuggerBreakpointCallback callback, gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 	
@@ -894,7 +973,7 @@ idebugger_breakpoint_add_at_function (IAnjutaDebuggerBreakpoint *plugin, const g
 }
 
 static gboolean
-idebugger_breakpoint_add_at_address (IAnjutaDebuggerBreakpoint *plugin, gulong address, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
+idebugger_breakpoint_add_at_address (IAnjutaDebuggerBreakpoint *plugin, gulong address, IAnjutaDebuggerBreakpointCallback callback, gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 	
@@ -904,7 +983,7 @@ idebugger_breakpoint_add_at_address (IAnjutaDebuggerBreakpoint *plugin, gulong a
 }
 
 static gboolean
-idebugger_breakpoint_enable (IAnjutaDebuggerBreakpoint *plugin, guint id, gboolean enable, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
+idebugger_breakpoint_enable (IAnjutaDebuggerBreakpoint *plugin, guint id, gboolean enable, IAnjutaDebuggerBreakpointCallback callback, gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 	
@@ -914,7 +993,7 @@ idebugger_breakpoint_enable (IAnjutaDebuggerBreakpoint *plugin, guint id, gboole
 }
 
 static gboolean
-idebugger_breakpoint_ignore (IAnjutaDebuggerBreakpoint *plugin, guint id, guint ignore, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
+idebugger_breakpoint_ignore (IAnjutaDebuggerBreakpoint *plugin, guint id, guint ignore, IAnjutaDebuggerBreakpointCallback callback, gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 	
@@ -924,7 +1003,7 @@ idebugger_breakpoint_ignore (IAnjutaDebuggerBreakpoint *plugin, guint id, guint
 }
 
 static gboolean
-idebugger_breakpoint_condition (IAnjutaDebuggerBreakpoint *plugin, guint id, const gchar *condition, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
+idebugger_breakpoint_condition (IAnjutaDebuggerBreakpoint *plugin, guint id, const gchar *condition, IAnjutaDebuggerBreakpointCallback callback, gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 	
@@ -934,7 +1013,7 @@ idebugger_breakpoint_condition (IAnjutaDebuggerBreakpoint *plugin, guint id, con
 }
 
 static gboolean
-idebugger_breakpoint_remove (IAnjutaDebuggerBreakpoint *plugin, guint id, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
+idebugger_breakpoint_remove (IAnjutaDebuggerBreakpoint *plugin, guint id, IAnjutaDebuggerBreakpointCallback callback, gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -944,7 +1023,7 @@ idebugger_breakpoint_remove (IAnjutaDebuggerBreakpoint *plugin, guint id, IAnjut
 }
 
 static gboolean
-idebugger_breakpoint_list (IAnjutaDebuggerBreakpoint *plugin, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
+idebugger_breakpoint_list (IAnjutaDebuggerBreakpoint *plugin, IAnjutaDebuggerGListCallback callback, gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -971,7 +1050,7 @@ idebugger_breakpoint_iface_init (IAnjutaDebuggerBreakpointIface *iface)
  *---------------------------------------------------------------------------*/
 
 static gboolean
-idebugger_register_list (IAnjutaDebuggerRegister *plugin, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
+idebugger_register_list (IAnjutaDebuggerRegister *plugin, IAnjutaDebuggerGListCallback callback , gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -981,7 +1060,7 @@ idebugger_register_list (IAnjutaDebuggerRegister *plugin, IAnjutaDebuggerCallbac
 }
 
 static gboolean
-idebugger_register_update (IAnjutaDebuggerRegister *plugin, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
+idebugger_register_update (IAnjutaDebuggerRegister *plugin, IAnjutaDebuggerGListCallback callback , gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -1012,7 +1091,7 @@ idebugger_register_iface_init (IAnjutaDebuggerRegisterIface *iface)
  *---------------------------------------------------------------------------*/
 
 static gboolean
-idebugger_memory_inspect (IAnjutaDebuggerMemory *plugin, gulong address, guint length, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
+idebugger_memory_inspect (IAnjutaDebuggerMemory *plugin, gulong address, guint length, IAnjutaDebuggerMemoryCallback callback , gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -1031,7 +1110,7 @@ idebugger_memory_iface_init (IAnjutaDebuggerMemoryIface *iface)
  *---------------------------------------------------------------------------*/
 
 static gboolean
-idebugger_instruction_disassemble (IAnjutaDebuggerInstruction *plugin, gulong address, guint length, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
+idebugger_instruction_disassemble (IAnjutaDebuggerInstruction *plugin, gulong address, guint length, IAnjutaDebuggerInstructionCallback callback , gpointer user_data, GError **err)
 {
 	GdbPlugin *this = (GdbPlugin *)plugin;
 
@@ -1070,6 +1149,16 @@ idebugger_instruction_run_to_address (IAnjutaDebuggerInstruction *plugin, gulong
 	return TRUE;
 }
 
+static gboolean
+idebugger_instruction_run_from_address (IAnjutaDebuggerInstruction *plugin, gulong address, GError **err)
+{
+	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
+	
+	debugger_run_from_address (this->debugger, address);
+
+	return TRUE;
+}
+
 static void
 idebugger_instruction_iface_init (IAnjutaDebuggerInstructionIface *iface)
 {
@@ -1077,6 +1166,7 @@ idebugger_instruction_iface_init (IAnjutaDebuggerInstructionIface *iface)
 	iface->step_in_instruction = idebugger_instruction_step_in;
 	iface->step_over_instruction = idebugger_instruction_step_over;
 	iface->run_to_address = idebugger_instruction_run_to_address;
+	iface->run_from_address = idebugger_instruction_run_from_address;
 }
 
 /* Implementation of IAnjutaDebuggerVariable interface
@@ -1086,14 +1176,17 @@ static gboolean
 idebugger_variable_destroy (IAnjutaDebuggerVariable *plugin, const gchar *name, GError **error)
 {
 	GdbPlugin *gdb = ANJUTA_PLUGIN_GDB (plugin);
+	gchar *quoted;
 
-	debugger_delete_variable (gdb->debugger, name);
+	quoted = quote_expression (name);
+	debugger_delete_variable (gdb->debugger, quoted);
+	g_free (quoted);
 
 	return TRUE;
 }
 
 static gboolean
-idebugger_variable_evaluate (IAnjutaDebuggerVariable *plugin, const gchar *name, IAnjutaDebuggerCallback callback , gpointer user_data, GError **error)
+idebugger_variable_evaluate (IAnjutaDebuggerVariable *plugin, const gchar *name, IAnjutaDebuggerGCharCallback callback , gpointer user_data, GError **error)
 {
 	GdbPlugin *gdb = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -1106,34 +1199,43 @@ static gboolean
 idebugger_variable_assign (IAnjutaDebuggerVariable *plugin, const gchar *name, const gchar *value, GError **error)
 {
 	GdbPlugin *gdb = ANJUTA_PLUGIN_GDB (plugin);
+	gchar *quoted;
 
-	debugger_assign_variable (gdb->debugger, name, value);
+	quoted = quote_expression (name);
+	debugger_assign_variable (gdb->debugger, quoted, value);
+	g_free (quoted);
 
 	return TRUE;
 }
 
 static gboolean
-idebugger_variable_list_children (IAnjutaDebuggerVariable *plugin, const gchar *name, IAnjutaDebuggerCallback callback , gpointer user_data, GError **error)
+idebugger_variable_list_children (IAnjutaDebuggerVariable *plugin, const gchar *name, guint from, IAnjutaDebuggerGListCallback callback , gpointer user_data, GError **error)
 {
 	GdbPlugin *gdb = ANJUTA_PLUGIN_GDB (plugin);
+	gchar *quoted;
 
-	debugger_list_variable_children (gdb->debugger, name, callback, user_data);
-
+	quoted = quote_expression (name);
+	debugger_list_variable_children (gdb->debugger, quoted, from, callback, user_data);
+	g_free (quoted);
+	
 	return TRUE;
 }
 
 static gboolean
-idebugger_variable_create (IAnjutaDebuggerVariable *plugin, const gchar *name, IAnjutaDebuggerCallback callback , gpointer user_data, GError **error)
+idebugger_variable_create (IAnjutaDebuggerVariable *plugin, const gchar *name, IAnjutaDebuggerVariableCallback callback , gpointer user_data, GError **error)
 {
 	GdbPlugin *gdb = ANJUTA_PLUGIN_GDB (plugin);
+	gchar *quoted;
 
-	debugger_create_variable (gdb->debugger, name, callback, user_data);
+	quoted = quote_expression (name);
+	debugger_create_variable (gdb->debugger, quoted, callback, user_data);
+	g_free (quoted);
 
 	return TRUE;
 }
 
 static gboolean
-idebugger_variable_update (IAnjutaDebuggerVariable *plugin, IAnjutaDebuggerCallback callback , gpointer user_data, GError **error)
+idebugger_variable_update (IAnjutaDebuggerVariable *plugin, IAnjutaDebuggerGListCallback callback , gpointer user_data, GError **error)
 {
 	GdbPlugin *gdb = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -1153,6 +1255,28 @@ idebugger_variable_iface_init (IAnjutaDebuggerVariableIface *iface)
 	iface->update = idebugger_variable_update;
 }
 
+/* Implementation of IAnjutaPreference interface
+ *---------------------------------------------------------------------------*/
+
+static void
+ipreferences_merge(IAnjutaPreferences* ipref, AnjutaPreferences* prefs, GError** error)
+{
+	gdb_merge_preferences (prefs, &(ANJUTA_PLUGIN_GDB (ipref)->pretty_printers));
+}
+
+static void
+ipreferences_unmerge(IAnjutaPreferences* ipref, AnjutaPreferences* prefs, GError** error)
+{
+	gdb_unmerge_preferences (prefs);
+}
+
+static void
+ipreferences_iface_init(IAnjutaPreferencesIface* iface)
+{
+	iface->merge = ipreferences_merge;
+	iface->unmerge = ipreferences_unmerge;
+}
+
 ANJUTA_PLUGIN_BEGIN (GdbPlugin, gdb_plugin);
 ANJUTA_PLUGIN_ADD_INTERFACE(idebugger, IANJUTA_TYPE_DEBUGGER);
 ANJUTA_PLUGIN_ADD_INTERFACE(idebugger_breakpoint, IANJUTA_TYPE_DEBUGGER_BREAKPOINT);
@@ -1160,6 +1284,7 @@ ANJUTA_PLUGIN_ADD_INTERFACE(idebugger_register, IANJUTA_TYPE_DEBUGGER_REGISTER);
 ANJUTA_PLUGIN_ADD_INTERFACE(idebugger_memory, IANJUTA_TYPE_DEBUGGER_MEMORY);
 ANJUTA_PLUGIN_ADD_INTERFACE(idebugger_instruction, IANJUTA_TYPE_DEBUGGER_INSTRUCTION);
 ANJUTA_PLUGIN_ADD_INTERFACE(idebugger_variable, IANJUTA_TYPE_DEBUGGER_VARIABLE);
+ANJUTA_PLUGIN_ADD_INTERFACE (ipreferences, IANJUTA_TYPE_PREFERENCES);
 ANJUTA_PLUGIN_END;
 
 ANJUTA_SIMPLE_PLUGIN (GdbPlugin, gdb_plugin);
